package LF::Sentence;

use LF::Sentence::Word;
use LF::Sentence::Relation;
use LF::Sentence::RelationArg;
use LF::Sentence::AbstractRelation;
use LF::Sentence::AbstractRelationArg;

use Data::Dumper;
use PerlLib::Collection;

use Class::MethodMaker
  new_with_init => 'new',
  get_set       => [

qw / Sentence Words Relations AbstractRelations Variables LogicForms /

];

sub init {
  my ($self,%args) = @_;
  $self->Sentence($args{Sentence});
  $self->Words
    (PerlLib::Collection->new
     (Type => "LF::Sentence::Word"));
  $self->Words->Contents({});
  $self->Relations
    (PerlLib::Collection->new
     (Type => "LF::Sentence::Relation"));
  $self->Relations->Contents({});
  $self->AbstractRelations
    (PerlLib::Collection->new
     (Type => "LF::Sentence::AbstractRelation"));
  $self->AbstractRelations->Contents({});
  $self->Variables
    (PerlLib::Collection->new
     (Type => "LF::Sentence::Variable"));
  $self->Variables->Contents({});
  $self->Parse;
}

sub Parse {
  my ($self,%args) = @_;
  foreach my $line (split /\n/, $self->Sentence) {
    if ($line =~ /\s/) {
      my @c = split /\s+/,$line;
      my @a = splice @c, 0, 5;
      my $predicate = shift @c;
      my $word1 = $self->FindOrCreate(Contents => \@a);
      my $word2 = $self->FindOrCreate(Contents => \@c);
      my $relation = LF::Sentence::Relation->new
	(
	 Predicate => $predicate,
	 Word1 => $word1,
	 Word2 => $word2,
	);
      # what to do with the preds
      $relation->ID
	($self->Relations->AddAutoIncrement
	 (Item => $relation));
      $word1->RelationArgs->AddAutoIncrement
	(Item => LF::Sentence::RelationArg->new
	 (
	  Relation => $relation,
	  ArgN => "0",
	 ));
      $word2->RelationArgs->AddAutoIncrement
	(Item => LF::Sentence::RelationArg->new
	 (
	  Relation => $relation,
	  ArgN => "1",
	 ));

      # now create the abstract relation if necessary
      if ($predicate =~ /^ARG(\d+)$/) {
	my $argnum = $1;
	# check whether a new abstractrelation exists

	# need to look at the wordid of the predicate of the abstract
	# relation

	my $ar;
	if (! exists $self->AbstractRelations->Contents->{$word1->ID}) {
	  $ar = LF::Sentence::AbstractRelation->new
	     (Predicate => $word1);
	  $self->AbstractRelations->Add
	    ($word1->ID => $ar);
	  $word1->AbstractRelation($ar);
	} else {
	  $ar = $self->AbstractRelations->Contents->{$word1->ID};
	}
	$ar->Args->[$argnum - 1] = $word2;
	$word2->AbstractRelationArgs->AddAutoIncrement
	  (Item => LF::Sentence::AbstractRelationArg->new
	   (
	    AbstractRelation => $ar,
	    ArgN => $argnum - 1,
	   ));
      }
    }
  }

  foreach my $word ($self->Words->Values) {
    $word->PopulateItemsAndEvents;
  }

  # now we can go ahead and generate the logic forms
  foreach my $word (sort {$a->ID <=> $b->ID} $self->Words->Values) {
    if (defined $word->SelfVar and $word->SelfVar->Type eq "item") {
      # this is a new variable
      $word->SelfVar->ID
	($self->Variables->AddAutoIncrement
	 (Item => $word->SelfVar));
    }
  }
  foreach my $word (sort {$a->ID <=> $b->ID} $self->Words->Values) {
    if (defined $word->SelfVar and $word->SelfVar->Type eq "event") {
      # this is a new variable
      $word->SelfVar->ID
	($self->Variables->AddAutoIncrement
	 (Item => $word->SelfVar));
    }
  }

  # now print out the forms
  my @forms;
  foreach my $word (sort {$a->ID <=> $b->ID} $self->Words->Values) {
    $word->GenerateLogicForm;
    if (defined $word->LogicForm) {
      push @forms, $word->LogicForm->Print;
    }
  }
  $self->LogicForms(\@forms);
}

sub FindOrCreate {
  my ($self,%args) = @_;
  my $id = $args{Contents}->[4];
  if (! exists $self->Words->Contents->{$id}) {
    my $word = LF::Sentence::Word->new(List => $args{Contents});
    $self->Words->Add($id => $word);
  }
  return $self->Words->Contents->{$id};
}

sub NonOOLogicForm {
  my ($self,%args) = @_;
  my $itemtable = [qw{orig stemmed origpos stemmedpos id}];
  my $itembase = {};
  my $vars = {};
  my $rels = {};
  my $mods = {};
  foreach my $item (split /\n/, $args{Input}) {
    if ($item =~ /\s/) {
      my $res = [];
      my @c = split /\s+/, $item;
      my @a = splice @c,0,5;
      $res->[0] = shift @c;
      my $i = 0;
      foreach my $record (@a) {
	$res->[1]->{$itemtable->[$i++]} = $record;
      }
      $i = 0;
      foreach my $record (@c) {
	$res->[2]->{$itemtable->[$i++]} = $record;
      }
      my $r = $res;

      # $rels->{$r->[0]}->{$r->[1]->{id}}->{$r->[2]->{id}} = 1;
      if ($r->[0] =~ /^ARG(\d+)$/) {
	$rels->{$r->[1]->{id}}->{$1} = $r->[2]->{id};
      } elsif ($r->[0] =~ /^MOD$/) {
	# we probably want to collapse these items
	$mods->{$r->[1]->{id}} = $r->[2]->{id};
      }
      for (my $i = 1; $i <= 2; ++$i) {
	# create the index of items for future reference
	$itembase->{$r->[$i]->{id}} = $r->[$i];
	if ($r->[$i]->{stemmedpos} =~ /^(VB)$/) {
	  $vars->{$r->[$i]->{id}} = {
				     "Type" => "event",
				     "Modifier" => "v_",
				    };
	} elsif ($r->[$i]->{stemmedpos} =~ /^(RB)$/) {
	  $vars->{$r->[$i]->{id}} = {
				     "Type" => "event",
				     "Modifier" => "r_",
				    };
	} elsif ($r->[$i]->{stemmedpos} =~ /^(TO|IN|CC)$/) {
	  $vars->{$r->[$i]->{id}} = {
				     "Type" => "event",
				    };
	} elsif ($r->[$i]->{stemmedpos} =~ /^(DT|-COMMA-)$/) {
	} elsif ($r->[$i]->{stemmedpos} =~ /^(NNP|NN|NNS)$/) {
	  $vars->{$r->[$i]->{id}} = {
				     Type => "item",
				     Modifier => "n_",
				    };
	} elsif ($r->[$i]->{stemmedpos} =~ /^(JJ)$/) {
	  $vars->{$r->[$i]->{id}} = {
				     Type => "item",
				     Modifier => "a_",
				    };
	} elsif ($r->[$i]->{stemmedpos} =~ /^(PRP)$/) {
	  $vars->{$r->[$i]->{id}} = {
				     Type => "item",
				    };
	}
      }
      # now populate the relation data
    }
  }
  # print Dumper([$itembase,$vars]);
  # generate the predicates
  my @res;
  my ($items, $events) = ({},{});
  my $i = 1;
  my $iitems = {};
  foreach my $id (sort {$a <=> $b} keys %$itembase) {
    if (exists $vars->{$id} and $vars->{$id}->{Type} eq "item") {
      $items->{$i} = {
		      ID => $id,
		      Place => $i,
		      Content => $itembase->{$id}->{orig}.
		      (defined $vars->{$id}->{Modifier} ? ":".$vars->{$id}->{Modifier} : ""),
		     };
      $iitems->{$id} = $i;
      ++$i;
    }
  }
  my $ievents = {};
  foreach my $id (sort {$a <=> $b} keys %$itembase) {
    if (exists $vars->{$id} and $vars->{$id}->{Type} eq "event") {
      $events->{$i} = {
		       ID => $id,
		       Place => $i,
		       Content => $itembase->{$id}->{stemmed}.
		       (defined $vars->{$id}->{Modifier} ? ":".$vars->{$id}->{Modifier} : ""),
		      };
      $ievents->{$id} = $i;
      ++$i;
    }
  }
  # now generate the logic form of this
  # the items
  foreach my $i (sort {$a <=> $b} keys %$items) {
    push @res, $items->{$i}->{Content}." (x".$i.")";
  }

  # now the forms
  foreach my $i (sort {$a <=> $b} keys %$events) {
    # these are relations
    # extract the relation and put the various
    my $string = $events->{$i}->{Content}." (e".$i;
    # now compute the args for this event and their item number
    # iterate over the arguments for this event
    my $j = 1;
    while (exists $rels->{$events->{$i}->{ID}}->{$j}) {
      my $num = $rels->{$events->{$i}->{ID}}->{$j};
      if (exists $iitems->{$num}) {
	$string .= ", x".$iitems->{$num};
      } elsif (exists $ievents->{$num}) {
	$string .= ", e".$ievents->{$num};
      }
      ++$j;
    }
    push @res, $string.")";
  }
  return \@res;
}

1;
