package System::Enju;

use LF;

# use Manager::Dialog qw(QueryUser);

use Data::Dumper;
use Expect;
use Lingua::EN::Sentence qw(get_sentences);

use Class::MethodMaker
  new_with_init => 'new',
  get_set       =>
  [

   qw / MyExpect MyLF /

  ];

sub init {
  my ($self,%args) = @_;
  $self->MyLF(LF->new);
}

sub StartEnju {
  my ($self,%args) = @_;
  if (! defined $self->MyExpect) {
    $self->StartServer(%args);
  }
}

sub StartServer {
  my ($self,%args) = @_;
  my $command = "enju";
  my @parameters;
  $self->MyExpect(Expect->new);
  $self->MyExpect->raw_pty(1);
  $self->MyExpect->spawn($command, @parameters)
    or die "Cannot spawn $command: $!\n";
  print "Waiting for parser to initialize...\n";
  $self->MyExpect->expect(300, [qr/Ready/, sub {print "Initialized.\n"}]);
  $self->MyExpect->clear_accum();
}

sub StopServer {
  my ($self,%args) = @_;
  if ($args{HardRestart}) {
    $self->MyExpect->hard_close();
  } else {
    $self->MyExpect->soft_close();
  }
}

sub RestartServer {
  my ($self,%args) = @_;
  $self->StopServer
    (HardRestart => 1);
  $self->StartServer;
}

sub ProcessSentence {
  my ($self,%args) = @_;
  my $sentence = $args{Sentence};
  $self->MyExpect->print("$sentence\n");
  $self->MyExpect->expect(300, [qr/./, sub {} ]);
  my $res = $self->MyExpect->match();
  $self->MyExpect->expect(300, [qr/^$/m, sub {} ]);
  $res .= $self->MyExpect->before();
  if (! $res) {
    # timed out, restart server
    $self->RestartServer;
    return {Fail => 1};
  } else {
    $self->MyExpect->clear_accum();
    return {Result => $res};
  }
}

sub ProcessFormalism {
  my ($self,%args) = @_;
  my $formalism = $args{Formalism};
  if ($formalism->{Fail}) {
    return $formalism;
  } else {
    if ($args{Type} eq "LogicForm") {
      return [Results => $self->MyLF->LogicFormOfSentence
	      (Input => $formalism->{Result})];
    } else {
      my $roots = {};
      my $root;
      foreach my $l (split /\n/, $formalism->{Result}) {
	next if $l =~ /^$/;
	next if $l =~ /^\s*\#/;
	# print $l."\n";
	my @i = split /\s+/,$l;
	$roots->{$i[1]}->{$i[5]} = $i[7];
      }

      my @formula;
      foreach my $key (keys %$roots) {
	my @args = map {$roots->{$key}->{$_}} sort keys %{$roots->{$key}};
	push @formula, "\t($key ".join (" ",@args).")";
      }
      return {Results => \@formula};
    }
  }
}

sub ApplyEnjuToSentence {
  my ($self,%args) = @_;
  my $sentence = $args{Sentence};
  $sentence =~ s/\n/ /g;
  $sentence =~ s/\s+/ /g;
  if (length($sentence) < 150) {
    print $sentence."\n";
    return $self->ProcessFormalism
      (Formalism => $self->ProcessSentence
       (Sentence => $sentence),
       Type => $args{Type});
  } else {
    return {SentenceTooLong => 1};
  }
}

sub ApplyEnjuToText {
  my ($self,%args) = @_;
  my $text = $args{Text};
  print "Getting sentences...\n";
  my $sentences = get_sentences($text);
  print "Done\n";
  my @results;
  foreach my $sentence (@$sentences) {
    push @results, $self->ProcessFormalism
      (Formalism => $self->ProcessSentence
       (Sentence => $sentence),
       Type => $args{Type});
  }
  return \@results;
}


sub ApplyEnjuToFile {
  my ($self,%args) = @_;
  my $f = $args{File};
  if (-f $f) {
    my $text = `cat "$f"`;
    return $self->ApplyEnjuToText
      (Type => $args{Type});
  } else {
    return {FileNotFound => 1};
  }
}

sub ApplyEnjuToQueryUser {
  my ($self,%args) = @_;
  while (1) {
    # my $sentence = QueryUser("Sentence?: ");
  }
}

1;
